@@ -14,6 +14,7 @@
 #' @param save_model `TRUE` or `FALSE` indicating whether to save the generated model.
 #' @param save_path path for saving the generated model.
 #' @param parallel `TRUE` or `FALSE` indicating whether to use parallelization in model training.
+#' @param n_cores number of cores to use in parallel model training (if `parallel` is `TRUE`).
 #' @param include_comp_inds `TRUE` or `FALSE` indicating whether to generate and use competition indices as covariates.
 #' @param competition_radius distance for competition radius if \code{include_comp_inds} is `TRUE`.
 #' @param correction type of correction to apply ("none", "toroidal", or "truncation").
@@ -22,7 +23,7 @@
 #' @param tuning_grid_size size of the tuning grid for hyperparameter tuning.
 #' @param verbose `TRUE` or `FALSE` indicating whether to show progress of model training.
 #'
-#' @return a list containing the raw trained model and a bundled model object.
+#' @return an ldmppr_mark_model object.
 #' @export
 #'
 #' @examples
@@ -41,7 +42,7 @@
 #'   dplyr::mutate(time = power_law_mapping(size, .5))
 #'
 #' # Train the model
-#' train_mark_model(
+#' mark_model <- train_mark_model(
 #'   data = locations,
 #'   raster_list = scaled_raster_list,
 #'   scaled_rasters = TRUE,
@@ -57,6 +58,8 @@
 #'   verbose = TRUE
 #' )
 #'
+#' print(mark_model)
+#'
 train_mark_model <- function(data,
                              raster_list = NULL,
                              scaled_rasters = FALSE,
@@ -65,6 +68,7 @@ train_mark_model <- function(data,
                              save_model = FALSE,
                              save_path = NULL,
                              parallel = TRUE,
+                             n_cores = NULL,
                              include_comp_inds = FALSE,
                              competition_radius = 15,
                              correction = "none",
@@ -76,12 +80,15 @@ train_mark_model <- function(data,
   if (is.null(raster_list) | !is.list(raster_list)) stop("Provide a list of rasters for the raster_list argument.", .call = FALSE)
   if (is.null(xy_bounds) | !(length(xy_bounds) == 4)) stop("Provide (x,y) bounds in the form (a_x, b_x, a_y, b_y) for the xy_bounds argument.", .call = FALSE)
   if (xy_bounds[2] < xy_bounds[1] | xy_bounds[4] < xy_bounds[3]) stop("Provide (x,y) bounds in the form (a_x, b_x, a_y, b_y) for the xy_bounds argument.", .call = FALSE)
-  if (save_model == TRUE & is.null(save_path)) stop("Provide a path for saving the bundled model object.", .call = FALSE)
+  if (isTRUE(save_model) && is.null(save_path)) stop("Provide a path for saving the mark model object.", .call = FALSE)
   if (!correction %in% c("none", "toroidal", "truncation")) stop("Provide a valid correction type for the correction argument.", .call = FALSE)
   if (include_comp_inds == TRUE & (is.null(competition_radius) | competition_radius < 0)) stop("Provide the desired radius for competition_indices argument.", .call = FALSE)
-  if (!selection_metric %in% c("rmse", "mae", "rsq")) stop("Provide a valid correction type for the correction argument.", .call = FALSE)
+  if (!selection_metric %in% c("rmse", "mae", "rsq")) stop("Provide a valid metric for selection_metric ('rmse', 'mae', or 'rsq').", .call = FALSE)
   if (!model_type %in% c("xgboost", "random_forest")) stop("Provide a valid model type for the model_type argument.", .call = FALSE)
   if (!is.logical(parallel)) stop("Provide a logical value for the parallel argument.", .call = FALSE)
+  if (parallel == TRUE & !is.null(n_cores)) {
+    if (!is.numeric(n_cores) | n_cores < 1) stop("Provide a numeric value greater than 0 for the n_cores argument.", .call = FALSE)
+  }
   if (!is.logical(include_comp_inds)) stop("Provide a logical value for the include_comp_inds argument.", .call = FALSE)
   if (!is.logical(save_model)) stop("Provide a logical value for the save_model argument.", .call = FALSE)
   if (!is.logical(verbose)) stop("Provide a logical value for the verbose argument.", .call = FALSE)
@@ -89,10 +96,22 @@ train_mark_model <- function(data,
   if (!is.numeric(tuning_grid_size) | tuning_grid_size < 1) stop("Provide a numeric value greater than 0 for the tuning_grid_size argument.", .call = FALSE)
   if (!is.logical(scaled_rasters)) stop("Provide a logical value for the scaled_rasters argument.", .call = FALSE)
 
-
   # Initialize parallelization for model training
-  if (parallel) {
-    doParallel::registerDoParallel()
+  cl <- NULL
+  if (isTRUE(parallel)) {
+    if( !is.null(n_cores)) {
+      n_workers <- n_cores
+    } else {
+      n_workers <- max(1L, floor(parallel::detectCores()/2))
+    }
+    cl <- parallel::makePSOCKcluster(n_workers)
+    doParallel::registerDoParallel(cl)
+    on.exit({
+      parallel::stopCluster(cl)
+      foreach::registerDoSEQ()
+    }, add = TRUE)
+  } else {
+    foreach::registerDoSEQ()
   }
 
   # Obtain a matrix of (x, y) locations
@@ -111,12 +130,12 @@ train_mark_model <- function(data,
 
   if (include_comp_inds == TRUE) {
     # Calculate competition indices in a given radius
-    X$near_nbr_dist <- NA
-    X$near_nbr_num <- NA
-    X$avg_nbr_dist <- NA
-    X$near_nbr_time <- NA
-    X$near_nbr_time_all <- NA
-    X$near_nbr_time_dist_ratio <- NA
+    X$near_nbr_dist <- NA_real_
+    X$near_nbr_num <- NA_real_
+    X$avg_nbr_dist <- NA_real_
+    X$near_nbr_time <- NA_real_
+    X$near_nbr_time_all <- NA_real_
+    X$near_nbr_time_dist_ratio <- NA_real_
 
     # Calculate distance matrices for selected correction method
     if ((correction == "none") | (correction == "truncation")) {
@@ -125,34 +144,37 @@ train_mark_model <- function(data,
       distance_matrix <- toroidal_dist_matrix_optimized(s, xy_bounds[2] - xy_bounds[1], xy_bounds[4] - xy_bounds[3])
     }
 
-
     for (i in 1:base::nrow(X)) {
       close_points <- base::unique(base::which(distance_matrix[i, ] < competition_radius & distance_matrix[i, ] != 0))
       close_times <- X$time[close_points]
+
       X$near_nbr_dist[i] <- base::min(distance_matrix[i, ][-i])
       X$near_nbr_num[i] <- base::length(close_points)
       X$avg_nbr_dist[i] <- base::mean(distance_matrix[i, ][close_points])
+
       if (base::length(close_points) == 0) {
         X$avg_nbr_dist[i] <- base::min(distance_matrix[i, ][-i])
       }
+
       X$near_nbr_time[i] <- X$time[base::unique(base::which(distance_matrix[i, ] == X$near_nbr_dist[i]))]
       X$near_nbr_time_all[i] <- mean(close_times)
+
       if (base::length(close_points) == 0) {
         X$near_nbr_time_all[i] <- X$time[base::unique(base::which(distance_matrix[i, ] == X$near_nbr_dist[i]))]
       }
+
       X$near_nbr_time_dist_ratio[i] <- X$near_nbr_time[i] / X$near_nbr_dist[i]
     }
   }
 
-
   ## Define the model data for fitting the size model
   model_data <- data.frame(size = data$size, X)
 
   if (correction == "truncation") {
     model_data <- model_data[(model_data$x > 15 &
-      model_data$x < (xy_bounds[2] - xy_bounds[1]) - 15 &
-      model_data$y > 15 &
-      model_data$y < (xy_bounds[4] - xy_bounds[3]) - 15), ]
+                                model_data$x < (xy_bounds[2] - xy_bounds[1]) - 15 &
+                                model_data$y > 15 &
+                                model_data$y < (xy_bounds[4] - xy_bounds[3]) - 15), ]
   }
 
   if (verbose) {
@@ -166,10 +188,10 @@ train_mark_model <- function(data,
     strata = size
   )
 
-  # Specify the recipe
+  # Specify the recipe (retain = TRUE makes the prepped object more robust across sessions)
   preprocessing_recipe <-
     recipes::recipe(size ~ ., data = rsample::training(data_split)) %>%
-    recipes::prep()
+    recipes::prep(retain = TRUE)
 
   # Generate the cross validation folds
   data_cv_folds <-
@@ -179,13 +201,26 @@ train_mark_model <- function(data,
     ) %>%
     rsample::vfold_cv(v = cv_folds)
 
+  # Define the metric set
+  metric_set <- if (selection_metric == "rsq") {
+    yardstick::metric_set(yardstick::rmse, yardstick::mae, yardstick::rsq)
+  } else {
+    yardstick::metric_set(yardstick::rmse, yardstick::mae)
+  }
+
+  # Define control object for tuning
+  ctrl <- tune::control_grid(
+    verbose = FALSE,
+    parallel_over = if (isTRUE(parallel)) "resamples" else NULL
+  )
+
 
   if (model_type == "xgboost") {
-    if (verbose) {
-      base::message("Training XGBoost model...")
-    }
+    if (verbose) base::message("Training XGBoost model...")
+
+    # Choose threads safely: if we're parallelizing resamples, force xgboost to 1 thread
+    nthread <- if (isTRUE(parallel)) 1L else max(1L, parallel::detectCores() - 1L)
 
-    # Specify the XGBoost model
     xgboost_model <-
       parsnip::boost_tree(
         mode = "regression",
@@ -194,10 +229,19 @@ train_mark_model <- function(data,
         tree_depth = hardhat::tune(),
         learn_rate = hardhat::tune(),
         loss_reduction = hardhat::tune()
-      ) %>%
-      parsnip::set_engine("xgboost", objective = "reg:squarederror")
+      )
+
+    xgboost_engine_args <- list(
+      objective = "reg:squarederror",
+      nthread   = nthread,
+      verbose   = 0
+    )
+
+    xgboost_model <- do.call(
+      parsnip::set_engine,
+      c(list(object = xgboost_model, engine = "xgboost"), xgboost_engine_args)
+    )
 
-    # Specify the XGBoost hyperparameters for tuning
     xgboost_params <-
       dials::parameters(
         dials::trees(),
@@ -207,116 +251,126 @@ train_mark_model <- function(data,
         dials::loss_reduction()
       )
 
-    # Create the tuning grid
     xgboost_grid <-
       dials::grid_space_filling(
         xgboost_params,
         size = tuning_grid_size
       )
 
-    # Establish the model fitting workflow
     xgboost_wf <-
       workflows::workflow() %>%
       workflows::add_model(xgboost_model) %>%
       workflows::add_formula(size ~ .)
 
-    # Perform the hyperparameter tuning
     xgboost_tuned <- tune::tune_grid(
       object = xgboost_wf,
       resamples = data_cv_folds,
       grid = xgboost_grid,
-      metrics = yardstick::metric_set(yardstick::rmse, yardstick::mae),
+      metrics = yardstick::metric_set(yardstick::rmse, yardstick::mae, yardstick::rsq),
       control = tune::control_grid(verbose = FALSE)
     )
 
-    # Obtain the best model by "mae"
     xgboost_best_params <- xgboost_tuned %>%
       tune::select_best(metric = selection_metric)
 
-    # Obtain the finalized model
     xgboost_model_final <- xgboost_model %>%
       tune::finalize_model(xgboost_best_params)
 
-    # Fit the model on the full training data
     mark_model <- xgboost_model_final %>%
       parsnip::fit(
         formula = size ~ .,
         data    = model_data
       )
+
   } else if (model_type == "random_forest") {
-    if (verbose) {
-      base::message("Training Random Forest model...")
-    }
-    # Specify the Ranger random forest model
+    if (verbose) base::message("Training Random Forest model...")
+
     rf_model <-
       parsnip::rand_forest(
         mode = "regression",
         trees = hardhat::tune(),
         min_n = hardhat::tune()
-      ) %>%
-      parsnip::set_engine("ranger")
+      )
+
+    rf_engine_args <- list(
+      num.threads = if (isTRUE(parallel)) 1L else max(1L, parallel::detectCores() - 1L)
+    )
+
+    rf_model <- do.call(
+      parsnip::set_engine,
+      c(list(object = rf_model, engine = "ranger"), rf_engine_args)
+    )
 
-    # Specify the random forest hyperparameters for tuning
     rf_params <-
       dials::parameters(
         dials::trees(),
         dials::min_n()
       )
 
-    # Create the tuning grid
     rf_grid <-
       dials::grid_space_filling(
         rf_params,
         size = tuning_grid_size
       )
 
-    # Establish the model fitting workflow
     rf_wf <-
       workflows::workflow() %>%
       workflows::add_model(rf_model) %>%
       workflows::add_formula(size ~ .)
 
-    # Perform the hyperparameter tuning
     rf_tuned <- tune::tune_grid(
       object = rf_wf,
       resamples = data_cv_folds,
       grid = rf_grid,
-      metrics = yardstick::metric_set(yardstick::rmse, yardstick::mae),
+      metrics = yardstick::metric_set(yardstick::rmse, yardstick::mae, yardstick::rsq),
       control = tune::control_grid(verbose = FALSE)
     )
 
-    # Obtain the best model by "mae"
     rf_best_params <- rf_tuned %>%
       tune::select_best(metric = selection_metric)
 
-    # Obtain the finalized model
     rf_model_final <- rf_model %>%
       tune::finalize_model(rf_best_params)
 
-    # Fit the model on the full training data
     mark_model <- rf_model_final %>%
       parsnip::fit(
         formula = size ~ .,
         data    = model_data
       )
+
   } else {
     stop("Please provide xgboost or random_forest argument for model_type.")
   }
 
-  if (verbose) {
-    base::message("Training complete!")
+  if (verbose) base::message("Training complete!")
+
+  # Identify engine + extract engine fit
+  engine <- if (model_type == "xgboost") "xgboost" else "ranger"
+  fit_engine <- mark_model$fit
+
+  # Derive feature_names in baked order (for stable prediction)
+  baked_train <- recipes::bake(preprocessing_recipe, new_data = rsample::training(data_split))
+  baked_train$size <- NULL
+  feature_names <- names(baked_train)
+
+  mm <- ldmppr_mark_model(
+    engine = engine,
+    fit_engine = fit_engine,
+    recipe = preprocessing_recipe,
+    outcome = "size",
+    feature_names = feature_names,
+    info = list(
+      model_type = model_type,
+      correction = correction,
+      include_comp_inds = include_comp_inds,
+      cv_folds = cv_folds
+    )
+  )
+
+  if (isTRUE(save_model)) {
+    save_mark_model(mm, save_path)
   }
 
-  # Bundle the trained model object
-  bundled_mod <-
-    mark_model %>%
-    bundle::bundle()
-
-  if (save_model == TRUE) {
-    # Save the bundled model object
-    base::saveRDS(bundled_mod, file = save_path)
-  }
-
-  # Return a list containing the raw model and bundled model
-  return(base::list(raw_model = mark_model, bundled_model = bundled_mod))
+  return(mm)
 }
+
