@@ -3,68 +3,75 @@ knitr::opts_chunk$set(
   collapse = TRUE,
   comment = "#>"
 )
+set.seed(90210)
 
 ## ----setup--------------------------------------------------------------------
 library(ldmppr)
+library(dplyr)
 
 ## ----showdata-----------------------------------------------------------------
-# Load the data
 data("small_example_data")
-
-# Display the first few rows of the dataset
 nrow(small_example_data)
 head(small_example_data)
 
-## ----estimate_sc--------------------------------------------------------------
-# Map the observed sizes to arrival times using the power_law_mapping function
+## ----estimate_process_parameters----------------------------------------------
+# Use the (x, y, size) form and let estimate_process_parameters() construct time via delta
 parameter_estimation_data <- small_example_data %>%
-  dplyr::mutate(time = power_law_mapping(size = size, delta = .5)) %>%
-  dplyr::select(time, x, y)
+  dplyr::select(x, y, size)
 
-# Define the grid values
+# Define the integration / grid values used by the likelihood approximation
 x_grid <- seq(0, 25, length.out = 5)
 y_grid <- seq(0, 25, length.out = 5)
-t_grid <- seq(0, 1, length.out = 5)
+t_grid <- seq(0, 1,  length.out = 5)
 
-# Define the parameter initialization values
-parameter_inits <- c(1.5, 8.5, .015, 1.5, 3.2, .75, 3, .08)
+# Parameter initialization values: (alpha1, beta1, gamma1, alpha2, beta2, alpha3, beta3, gamma3)
+parameter_inits <- c(1.5, 8.5, 0.015, 1.5, 3.2, 0.75, 3, 0.08)
 
-# Define the upper bounds
+# Upper bounds for (t, x, y)
 upper_bounds <- c(1, 25, 25)
 
-# Estimate the parameters
-estimated_sc <- estimate_parameters_sc(
+
+fit_sc <- estimate_process_parameters(
   data = parameter_estimation_data,
+  process = "self_correcting",
   x_grid = x_grid,
   y_grid = y_grid,
   t_grid = t_grid,
-  parameter_inits = parameter_inits,
   upper_bounds = upper_bounds,
-  opt_algorithm = "NLOPT_LN_SBPLX",
-  nloptr_options = list(
-    maxeval = 300,
-    xtol_rel = 1e-3
-  ),
+  parameter_inits = parameter_inits,
+  delta = 1,
+  parallel = FALSE,
+  strategy = c("global_local"),
+  global_algorithm = "NLOPT_GN_CRS2_LM",
+  local_algorithm = "NLOPT_LN_BOBYQA",
+  global_options = list(maxeval = 150),
+  local_options = list(maxeval = 300, xtol_rel = 1e-5, maxtime = NULL),
   verbose = FALSE
 )
 
-# Obtain the optimal parameter estimates
-optimal_parameters <- estimated_sc$solution
-print(optimal_parameters)
+# Print method for ldmppr_fit objects
+print(fit_sc)
+
+# Extract parameters
+estimated_parameters <- coef(fit_sc)
+estimated_parameters
 
 ## ----train_mark_model---------------------------------------------------------
-# Load the example raster files
-raster_paths <- list.files(system.file("extdata", package = "ldmppr"), pattern = "\\.tif$", full.names = TRUE)
+# Load raster covariates shipped with the package
+raster_paths <- list.files(system.file("extdata", package = "ldmppr"),
+  pattern = "\\.tif$", full.names = TRUE
+)
 raster_paths <- raster_paths[!grepl("_med\\.tif$", raster_paths)]
 rasters <- lapply(raster_paths, terra::rast)
+
+# Scale rasters once up front
 scaled_rasters <- scale_rasters(rasters)
 
-# Generate the time values using the power law mapping with optimal delta
+# Create training data with a time column (consistent with the delta used above)
 model_training_data <- small_example_data %>%
-  dplyr::mutate(time = power_law_mapping(size, .5))
+  mutate(time = power_law_mapping(size, delta = 1))
 
-# Train the model
-example_mark_model <- train_mark_model(
+mark_model <- train_mark_model(
   data = model_training_data,
   raster_list = scaled_rasters,
   scaled_rasters = TRUE,
@@ -80,33 +87,36 @@ example_mark_model <- train_mark_model(
   verbose = TRUE
 )
 
-# Unbundle the model
-example_mark_model <- bundle::unbundle(example_mark_model$bundled_model)
+# S3 print method
+mark_model
+
+## ----save_load_mark_model, eval=FALSE-----------------------------------------
+# save_path <- tempfile(fileext = ".rds")
+# save_mark_model(mark_model, save_path)
+# 
+# mark_model2 <- load_mark_model(save_path)
+# mark_model2
 
 ## ----check_model_fit, fig.width=9, fig.height=9-------------------------------
-# Generate the reference pattern
-reference_data <- generate_mpp(
+# Create a marked point pattern from the observed data
+reference_mpp <- generate_mpp(
   locations = small_example_data[, c("x", "y")],
   marks = small_example_data$size,
   xy_bounds = c(0, 25, 0, 25)
 )
 
-# Define an anchor point
+# Anchor point (condition on an observed location)
 M_n <- as.matrix(small_example_data[1, c("x", "y")])
 
-# Specify the estimated parameters of the self-correcting process
-estimated_parameters <- optimal_parameters
-
-# Check the model fit
-example_model_fit <- check_model_fit(
-  reference_data = reference_data,
+model_check <- check_model_fit(
+  reference_data = reference_mpp,
   t_min = 0,
   t_max = 1,
   sc_params = estimated_parameters,
   anchor_point = M_n,
   raster_list = scaled_rasters,
   scaled_rasters = TRUE,
-  mark_model = example_mark_model,
+  mark_model = mark_model,
   xy_bounds = c(0, 25, 0, 25),
   include_comp_inds = TRUE,
   thinning = TRUE,
@@ -118,19 +128,18 @@ example_model_fit <- check_model_fit(
   seed = 90210
 )
 
-# Plot the combined global envelope test results
-plot(example_model_fit$combined_env)
+# S3 plot method (combined global envelope test)
+plot(model_check)
 
 ## ----simulate_mpp, fig.width=6.5, fig.height=6--------------------------------
-# Simulate a marked point process
-simulated_mpp <- simulate_mpp(
+simulated <- simulate_mpp(
   sc_params = estimated_parameters,
   t_min = 0,
   t_max = 1,
   anchor_point = M_n,
   raster_list = scaled_rasters,
   scaled_rasters = TRUE,
-  mark_model = example_mark_model,
+  mark_model = mark_model,
   xy_bounds = c(0, 25, 0, 25),
   include_comp_inds = TRUE,
   competition_radius = 10,
@@ -138,9 +147,9 @@ simulated_mpp <- simulate_mpp(
   thinning = TRUE
 )
 
-# Plot the simulated marked point process
-plot_mpp(
-  mpp_data = simulated_mpp$mpp,
-  pattern_type = "simulated"
-)
+# Plot method for ldmppr_sim
+plot(simulated)
+
+# Data-frame view of the simulated realization
+head(as.data.frame(simulated))
 
